package com.flutter_dhl_amap;

import android.app.Activity;
import android.content.Context;

import com.amap.api.maps.CoordinateConverter;
import com.amap.api.maps.model.LatLng;
import com.flutter_dhl_amap.Class.ConstraintsMap;
import com.flutter_dhl_amap.Class.PermissionUtils;
import com.flutter_dhl_amap.Location.DHL_LocationModel;
import com.flutter_dhl_amap.Search.DHL_DistanceSearch;

import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlutterDhlAmapPlugin */
public class FlutterDhlAmapPlugin implements MethodCallHandler {

  private EventChannel.EventSink eventSink = null;
  private  EventChannel.StreamHandler streamHandler= new EventChannel.StreamHandler() {
    @Override
    public void onListen(Object o, EventChannel.EventSink eventSink1) {
      eventSink = eventSink1;
    }

    @Override
    public void onCancel(Object o) {
      eventSink = null;
    }
  };

  private EventChannel.EventSink eventSink_Search = null;

  private  EventChannel.StreamHandler streamHandler_Search= new EventChannel.StreamHandler() {
    @Override
    public void onListen(Object o, EventChannel.EventSink eventSink1) {
      eventSink_Search = eventSink1;
    }

    @Override
    public void onCancel(Object o) {
      eventSink_Search = null;
    }
  };


  private Registrar registrar;
  private MethodChannel channel;
  private DHL_LocationModel dhl_locationModel;
  private DHL_DistanceSearch dhl_distanceSearch;
  public FlutterDhlAmapPlugin(Registrar registrar, MethodChannel channel) {
    this.registrar = registrar;
    this.channel = channel;

    EventChannel eventChannel = new EventChannel(registrar.messenger(),"flutter_dhl_amap_location");
    eventChannel.setStreamHandler(streamHandler);

    EventChannel eventChannel1 = new EventChannel(registrar.messenger(),"flutter_dhl_amap_Search");
    eventChannel1.setStreamHandler(streamHandler_Search);


  }

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_dhl_amap");
    channel.setMethodCallHandler(new FlutterDhlAmapPlugin(registrar,channel));



  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else if(call.method.equals("init")){

      PermissionUtils.requestPermissions(this,
              new String[]{
                      "android.permission.ACCESS_COARSE_LOCATION",
                      "android.permission.ACCESS_FINE_LOCATION",
                      "android.permission.ACCESS_NETWORK_STATE",
                      "android.permission.INTERNET",
                      "android.permission.WRITE_EXTERNAL_STORAGE",
                      "android.permission.ACCESS_LOCATION_EXTRA_COMMANDS",
                      "android.permission.BLUETOOTH",
                      "android.permission.BLUETOOTH_ADMIN",
                      "android.permission.CHANGE_WIFI_STATE",
                      "ndroid.permission.READ_PHONE_STATE",
                      "android.permission.ACCESS_WIFI_STATE",

              },

              new PermissionUtils.Callback() {
        @Override
        public void invoke(String[] permissions, int[] grantResults) {

        }
      });
    }else if(call.method.equals("start_location")){

      Map<String,Object> data = (Map<String, Object>) call.arguments;
      dhl_locationModel = new DHL_LocationModel(getApplicationContext(),data,eventSink);

    } else if(call.method.equals("stop_location")){
      dhl_locationModel.stop_location();

    }else if(call.method.equals("zhuan_location")){
      Map<String,Object> data = (Map<String, Object>) call.arguments;
      ConstraintsMap map = new ConstraintsMap();
      ConstraintsMap map1 = new ConstraintsMap();
      map.merge(data);
      String type= map.getString("type");
      Double latitude= map.getDouble("latitude");
      Double longitude= map.getDouble("longitude");
      if(latitude != 0 && longitude!=0){
        CoordinateConverter.CoordType type1 = CoordinateConverter.CoordType.GOOGLE;

        if(!type.equals("gaode")){
          type1= CoordinateConverter.CoordType.BAIDU;
        }
        CoordinateConverter converter  = new CoordinateConverter(getApplicationContext());
// CoordType.GPS 待转换坐标类型
        converter.from(type1);
// sourceLatLng待转换坐标点 LatLng类型
        converter.coord(new LatLng(latitude,longitude));
// 执行转换操作
        LatLng desLatLng = converter.convert();
        latitude = desLatLng.latitude;
        longitude = desLatLng.longitude;

      } else{
        latitude = 0.0;
        longitude = 0.0;
      }
      map1.putDouble("latitude",latitude);
      map1.putDouble("longitude",longitude);
      result.success(map1.toMap());
    }
    else if(call.method.equals("jisuan_location")){
      Map<String,Object> data = (Map<String, Object>) call.arguments;
      ConstraintsMap map = new ConstraintsMap();
      map.merge(data);

      if(dhl_distanceSearch == null){
        dhl_distanceSearch = new DHL_DistanceSearch(getApplicationContext(),eventSink_Search);
      }
      dhl_distanceSearch.jisuan(map);
    }
    else {
      result.notImplemented();
    }
  }

  public Activity getActivity(){
    return registrar.activity();
  }
  public Context getApplicationContext(){
    return registrar.activity().getApplicationContext();
  }

}
