package com.flutter_dhl_amap.Amap;

import android.content.Context;
import android.location.Location;
import android.os.Bundle;
import android.util.ArrayMap;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.amap.api.maps.AMap;
import com.amap.api.maps.AMapOptions;
import com.amap.api.maps.CameraUpdate;
import com.amap.api.maps.CameraUpdateFactory;
import com.amap.api.maps.TextureMapView;
import com.amap.api.maps.UiSettings;
import com.amap.api.maps.model.BitmapDescriptorFactory;
import com.amap.api.maps.model.CameraPosition;
import com.amap.api.maps.model.LatLng;
import com.amap.api.maps.model.Marker;
import com.amap.api.maps.model.MarkerOptions;
import com.amap.api.maps.model.MyLocationStyle;
import com.amap.api.maps.model.TextOptions;
import com.flutter_dhl_amap.Class.ConstraintsMap;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;

public class MyView implements PlatformView, MethodChannel.MethodCallHandler {

    private final MethodChannel methodChannel;
    private final TextureMapView mapView;
    private MyLocationStyle myLocationStyle;
    private UiSettings mUiSettings;//控件交互
    TextOptions textOptions;
    private AMap aMap;


    MyView(Context context, BinaryMessenger messenger, int id, Map<String, Object> params) {
        TextureMapView mapView = new TextureMapView(context);
        mapView.onCreate(new Bundle());
        this.mapView = mapView;

        ConstraintsMap constraintsMap = new ConstraintsMap( );
        constraintsMap.merge( params);

        Log.i("top",params.toString());

        aMap = this.mapView.getMap();

        aMap.setOnMyLocationChangeListener(new AMap.OnMyLocationChangeListener() {
            @Override
            public void onMyLocationChange(Location location) {
                //监听用户位置

            }
        });
        mUiSettings = aMap.getUiSettings();//实例化UiSettings类对象


        mUiSettings.setZoomControlsEnabled(constraintsMap.getBoolean("setZoomControlsEnabled"));//缩放按钮
        mUiSettings.setCompassEnabled(constraintsMap.getBoolean("setCompassEnabled"));//指南针

        mUiSettings.setMyLocationButtonEnabled(constraintsMap.getBoolean("setMyLocationButtonEnabled"));//定位按钮
        mUiSettings.setScaleControlsEnabled(constraintsMap.getBoolean("setScaleControlsEnabled"));//比例尺控件


        mUiSettings.setLogoPosition(AMapOptions.LOGO_POSITION_BOTTOM_LEFT);//设置logo位置

        mUiSettings.setZoomGesturesEnabled(constraintsMap.getBoolean("setZoomGesturesEnabled"));//缩放手势
        mUiSettings.setScrollGesturesEnabled(constraintsMap.getBoolean("setScrollGesturesEnabled"));//滑动手势
        mUiSettings.setRotateGesturesEnabled(constraintsMap.getBoolean("setRotateGesturesEnabled"));//旋转手势
        mUiSettings.setTiltGesturesEnabled(constraintsMap.getBoolean("setTiltGesturesEnabled"));//倾斜手势
        aMap.getUiSettings().setGestureScaleByMapCenter(constraintsMap.getBoolean("setGestureScaleByMapCenter"));//指定屏幕中心点的手势操作


        aMap.setMaxZoomLevel(constraintsMap.getInt("setMaxZoomLevel"));//最大缩放
        aMap.setMinZoomLevel(constraintsMap.getInt("setMinZoomLevel"));//最小缩放
        myLocationStyle= new MyLocationStyle();
        myLocationStyle.interval(constraintsMap.getInt("interval"));//设置连续定位模式下的定位间隔，只在连续定位模式下生效，单次定位模式下不会生效。单位为毫秒。
        myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_LOCATION_ROTATE_NO_CENTER);
        myLocationStyle.showMyLocation(constraintsMap.getBoolean("showMyLocation"));//设置是否显示定位小蓝点，用于满足只想使用定位，不想使用定位小蓝点的场景，设置false以后图面上不再有定位蓝点的概念，但是会持续回调位置信息。
        aMap.setMyLocationStyle(myLocationStyle);
        aMap.setMyLocationEnabled(constraintsMap.getBoolean("setMyLocationEnabled"));// 设置为true表示启动显示定位蓝点，false表示隐藏定位蓝点并不进行定位，默认是false。

        methodChannel = new MethodChannel(messenger, "Dhl_MyAmap_View_" + id);
        methodChannel.setMethodCallHandler(this);

        if(  constraintsMap.getBoolean("IsMyMarker")){

            MarkerOptions markerOption = new MarkerOptions().icon(BitmapDescriptorFactory
                    .defaultMarker(BitmapDescriptorFactory.HUE_AZURE))
                    .position(latLng)
                    .title("标题")
                    .snippet("详细信息")
                    .draggable(true);
            Mymarker = aMap.addMarker(markerOption);

            Mymarker.showInfoWindow();
        }


    }


    @Override
    public View getView() {
        return mapView;
    }

    @Override
    public void dispose() {
        mapView.onDestroy();
    }
    LatLng latLng;
    Marker Mymarker;
    Map<String ,Marker > markerList = new HashMap<String ,Marker >();
    @Override
    public void onMethodCall(MethodCall methodCall, MethodChannel.Result result) {
        ConstraintsMap constraintsMap = new ConstraintsMap( );
        Map<String,Object> data = (Map<String, Object>) methodCall.arguments;
        constraintsMap.merge(data);
        switch (methodCall.method) {
            case "setText":
                Log.i("top", "aaaaaaaaaaaaaaaaaaaaaaaa");
                break;
            case "SetCoordinate": //设置当前坐标

                latLng = new LatLng( constraintsMap.getDouble("latitude"),constraintsMap.getDouble("longitude"));//构造一个位置
                aMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng,16));
                if(Mymarker != null){
                    Mymarker.setPosition(latLng);
                }

                break;
            case "SetMymarkerContent": //设置我自己的坐标 内容
                if(Mymarker != null) {//我的标示 不能为null
                    String title = constraintsMap.getString("title");
                    String snippet = constraintsMap.getString("snippet");
                    if (title != "") Mymarker.setTitle(title);
                    if (snippet != "") Mymarker.setSnippet(snippet);

                }
                break;
            case "AddMarker":
                String key = constraintsMap.getString("key");
                double  latitude = constraintsMap.getDouble("latitude");
                double  longitude = constraintsMap.getDouble("longitude");

                LatLng latLng2 = new LatLng( latitude,longitude);//构造一个位置

                String title = constraintsMap.getString("title");
                String snippet = constraintsMap.getString("snippet");
                boolean is_show = constraintsMap.getBoolean("is_show");
                MarkerOptions markerOption = new MarkerOptions().icon(BitmapDescriptorFactory
                        .defaultMarker(BitmapDescriptorFactory.HUE_AZURE))
                        .position(latLng2)
                        .title(title)
                        .snippet(snippet)
                        .draggable(true);
                Marker marker = aMap.addMarker(markerOption);
                if(is_show){
                    marker.showInfoWindow();
                }
                markerList.put(key,marker);
                break;
            case "UpdateMarker"://更新坐标点到数据
                String Update_key = constraintsMap.getString("key");
                double  Update_latitude = constraintsMap.getDouble("latitude");
                double  Update_longitude = constraintsMap.getDouble("longitude");
                String Update_title = constraintsMap.getString("title");
                String Update_snippet = constraintsMap.getString("snippet");

                Marker Update_marker = markerList.get(Update_key);
                if(Update_marker == null )return;
                if (Update_latitude != 0 && Update_longitude != 0 ) {

                    LatLng latLng3 = new LatLng( Update_latitude,Update_longitude);//构造一个位置
                    Update_marker.setPosition(latLng3);
                }
                if (Update_title != "") Update_marker.setTitle(Update_title);
                if (Update_snippet != "") Update_marker.setSnippet(Update_snippet);

                break;
            case "UpdateMarkerShow"://更新坐标点到信息框
                String UpdateMarkerShow_key = constraintsMap.getString("key");
                boolean UpdateMarker_is_show = constraintsMap.getBoolean("is_show");
                Marker UpdateMarkerShow_marker = markerList.get(UpdateMarkerShow_key);
                if(UpdateMarkerShow_marker == null )return;
                if(UpdateMarker_is_show){
                    UpdateMarkerShow_marker.showInfoWindow();
                }else{
                    UpdateMarkerShow_marker.hideInfoWindow();
                }
                break;

            case "DelMarkerShow"://更新坐标点到信息框
                String DelMarkerShow_key = constraintsMap.getString("key");

                Marker DelMarkerShow_marker = markerList.get(DelMarkerShow_key);
                if(DelMarkerShow_marker == null )return;
                DelMarkerShow_marker.remove();
                markerList.remove(DelMarkerShow_key);
                break;

            case "DelAllMarkerShow"://删除所有坐标点

//                for (Map.Entry<String, Marker> entry : markerList.entrySet()) {
//                    Marker DelAllMarkerShow_marker =  entry.getValue();
//                    DelAllMarkerShow_marker.remove();
//                }
//
//
//                markerList =  new HashMap<String ,Marker >();

                break;
            default:
                result.notImplemented();
        }
    }
}