//
//  DHLAMapViewController.m
//  flutter_dhl_amap
//
//  Created by admin on 2019/6/30.
//

#import "DHLAMapViewController.h"
#import <MAMapKit/MAMapKit.h>
#import <AMapFoundationKit/AMapFoundationKit.h>

@interface DHLAMapViewController ()
@property(nonatomic)MAMapView* mapView;
@property(nonatomic)FlutterMethodChannel* channel;

@property(nonatomic)MAPointAnnotation *MypointAnnotation;
@property(nonatomic)NSMutableDictionary *markerList;



@end
@implementation DHLAMapViewController

- (instancetype)initWithWithFrame:(CGRect)frame
                   viewIdentifier:(int64_t)viewId
                        arguments:(id _Nullable)args
                  binaryMessenger:(NSObject<FlutterBinaryMessenger>*)messenger{
    
    _markerList = [[NSMutableDictionary alloc]init];
    self.mapView = [[MAMapView alloc] initWithFrame:self.view.bounds];
    NSDictionary *dic = args;
    self.mapView.showsUserLocation = [dic[@"showMyLocation"] boolValue] ;//小k蓝点
    self.mapView.userTrackingMode = MAUserTrackingModeFollow;
    self.mapView.showsCompass= [dic[@"setCompassEnabled"] boolValue] ;//指南针
    self.mapView.showsScale=  [dic[@"setScaleControlsEnabled"] boolValue] ;  //设置成NO表示不显示比例尺；YES表示显示比例尺
    self.mapView.zoomEnabled = [dic[@"setZoomGesturesEnabled"] boolValue];    //NO表示禁用缩放手势，YES表示开启
    self.mapView.scrollEnabled = [dic[@"setScrollGesturesEnabled"] boolValue];    //NO表示禁用滑动手势，YES表示开启
    self.mapView.rotateEnabled= [dic[@"setRotateGesturesEnabled"] boolValue];    //NO表示禁用旋转手势，YES表示开启
    self.mapView.rotateCameraEnabled= [dic[@"setTiltGesturesEnabled"] boolValue];    //NO表示禁用倾斜手势，YES表示开启
    [self.mapView setMaxZoomLevel:[ dic[@"setMaxZoomLevel"] floatValue]];
    [self.mapView setMinZoomLevel:[ dic[@"setMinZoomLevel"] floatValue]];
    
    MAUserLocationRepresentation *r = [[MAUserLocationRepresentation alloc] init];
    r.showsAccuracyRing = NO;///精度圈是否显示，默认YES
    r.showsHeadingIndicator = YES;///是否显示方向指示(MAUserTrackingModeFollowWithHeading模式开启)。默认为YES
//    r.fillColor = [UIColor redColor];///精度圈 填充颜色, 默认 kAccuracyCircleDefaultColor
//    r.strokeColor = [UIColor blueColor];///精度圈 边线颜色, 默认 kAccuracyCircleDefaultColor
//    r.lineWidth = 2;///精度圈 边线宽度，默认0
    r.enablePulseAnnimation = NO;///内部蓝色圆点是否使用律动效果, 默认YES
//    r.locationDotBgColor = [UIColor greenColor];///定位点背景色，不设置默认白色
//    r.locationDotFillColor = [UIColor grayColor];///定位点蓝色圆点颜色，不设置默认蓝色
//
    [_mapView updateUserLocationRepresentation:r];

    
    _mapView.delegate = self;
    if([dic[@"IsMyMarker"] boolValue]){
        _MypointAnnotation= [[MAPointAnnotation alloc] init];
        _MypointAnnotation.coordinate = CLLocationCoordinate2DMake(39.989631, 116.481018);
        _MypointAnnotation.title = @"标题";
        _MypointAnnotation.subtitle = @"详细信息";
       
        
        [_mapView addAnnotation:_MypointAnnotation];
        
    }
 
    NSString* channelName = [NSString stringWithFormat:@"Dhl_MyAmap_View_%lld", viewId];
    _channel = [FlutterMethodChannel methodChannelWithName:channelName binaryMessenger:messenger];
    __weak __typeof__(self) weakSelf = self;
    [_channel setMethodCallHandler:^(FlutterMethodCall *  call, FlutterResult  result) {
        [weakSelf onMethodCall:call result:result];
    }];
    
     return self;
}
 
- (MAAnnotationView *)mapView:(MAMapView *)mapView viewForAnnotation:(id <MAAnnotation>)annotation
{
    if ([annotation isKindOfClass:[MAUserLocation class]]) {
        
        return nil;
    }
    if ([annotation isKindOfClass:[MAPointAnnotation class]])
    {
        static NSString *pointReuseIndentifier = @"pointReuseIndentifier";
        MAPinAnnotationView*annotationView = (MAPinAnnotationView*)[mapView dequeueReusableAnnotationViewWithIdentifier:pointReuseIndentifier];
        if (annotationView == nil)
        {
            annotationView = [[MAPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pointReuseIndentifier];
        }
        
        annotationView.canShowCallout= YES;       //设置气泡可以弹出，默认为NO
        annotationView.animatesDrop = YES;        //设置标注动画显示，默认为NO
        annotationView.draggable = NO;        //设置标注可以拖动，默认为NO
        annotationView.canShowCallout = YES;
        annotationView.selected =YES;
        
        annotationView.pinColor = MAPinAnnotationColorPurple;
        
        
        return annotationView;
    }
    
    return nil;
}



-(void)onMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result{
    NSDictionary *dic = call.arguments;
    if ([[call method] isEqualToString:@"SetCoordinate"]) {
        
           CLLocationCoordinate2D latLng = CLLocationCoordinate2DMake([dic[@"latitude"]  floatValue] , [dic[@"longitude"]  floatValue] );
        [_MypointAnnotation setCoordinate:latLng];
        if(_MypointAnnotation != NULL){
            _MypointAnnotation.coordinate =latLng;
        }
        
        
    }else if ([[call method] isEqualToString:@"SetMymarkerContent"]){
        if(_MypointAnnotation != NULL){
            NSString *title = dic[@"title"] ;
            NSString *snippet = dic[@"snippet"] ;
            if(![title  isEqual: @""])_MypointAnnotation.title = title;
            if(![snippet  isEqual: @""])_MypointAnnotation.subtitle = snippet;
        }
        
    }else if([[call method] isEqualToString:@"AddMarker"]){
        NSString *key = dic[@"key"] ;
        
        CLLocationCoordinate2D latLng = CLLocationCoordinate2DMake([dic[@"latitude"]  floatValue] , [dic[@"longitude"]  floatValue] );

        NSString *title = dic[@"title"] ;
        NSString *snippet = dic[@"snippet"] ;
        Boolean is_show = [dic[@"is_show"] boolValue];
        
        MAPointAnnotation *MypointAnnotation= [[MAPointAnnotation alloc] init];
        
        MypointAnnotation.coordinate = latLng;
        MypointAnnotation.title = title;
        MypointAnnotation.subtitle = snippet;
        
        
        [_mapView addAnnotation:MypointAnnotation];
        [_markerList setValue:MypointAnnotation forKey:key];
        
    }else if([[call method] isEqualToString:@"UpdateMarker"]){
        NSString *key = dic[@"key"] ;
        MAPointAnnotation *MypointAnnotation= _markerList[key];
        if(MypointAnnotation == NULL)return;
        
        CLLocationDegrees latitude = [dic[@"latitude"]  floatValue];
        CLLocationDegrees longitude = [dic[@"longitude"]  floatValue];
        if(latitude!=0 && longitude!=0){
            CLLocationCoordinate2D latLng = CLLocationCoordinate2DMake(latitude, longitude);
            MypointAnnotation.coordinate = latLng;
        }
        NSString *title = dic[@"title"] ;
        NSString *snippet = dic[@"snippet"] ;
        if(![title  isEqual: @""])MypointAnnotation.title = title;
        if(![snippet  isEqual: @""])MypointAnnotation.subtitle = snippet;
        
    }else if([[call method] isEqualToString:@"UpdateMarkerShow"]){
        NSString *key = dic[@"key"] ;
        MAPointAnnotation *MypointAnnotation= _markerList[key];
        if(MypointAnnotation == NULL)return;
        
        
        
    }else if([[call method] isEqualToString:@"DelMarkerShow"]){
        NSString *key = dic[@"key"] ;
        MAPointAnnotation *MypointAnnotation= _markerList[key];
        if(MypointAnnotation == NULL)return;
        [_mapView removeAnnotation:MypointAnnotation];
        [_markerList removeObjectForKey:key];
        
    }
    else {
        result(FlutterMethodNotImplemented);
    }
}






- (nonnull UIView *)view {
    return self.mapView ;
}

@end
